import subprocess
import json
import requests
import sys
from requests.auth import HTTPBasicAuth
from urllib.parse import urlparse


def requestSettings(section, oldPlugin = False):
    """
    Запросить настройки через утилиту currentsettings
    :param section: название секции ini настроек или название старого плагина
    :param oldPlugin: запросить настройки для старого плагина
    :return: справочник с настройками
    """
    process = subprocess.Popen(["sudo", "/linuxcash/cash/bin/currentsettings", "-p" if oldPlugin else "-s", section],
                               stdout=subprocess.PIPE)
    process.wait()
    data, err = process.communicate()
    return json.loads(data)


def checkLogical(value):
    """
    Проверить логическое значение из настроек
    """
    return value in ["ok", "yes", "enable", "1", "true", "allow", "on", "y", "да"]


def getValue(settings, key):
    """
    Получить занчение настройки
    :param settings: справочник с настройками
    :param key: ключ
    :return: None если ключ отсутствует в справочнике, иначе - значение настройки
    """
    lowCaseDict = dict((k.lower(), v) for k, v in settings.items())
    if key.lower() not in lowCaseDict.keys():
        return None
    return lowCaseDict[key.lower()]


def pluginEnabled(pluginName):
    """
    Проверить включен ли плагин
    :param pluginName: название плагина
    :return: true - плагин включен, false - выключен
    """
    settings = requestSettings("plugins")
    return checkLogical(getValue(settings, pluginName))


def parseArtixBonusSettings(paresedXml):
    """
    Привести распаршенную XML-ку к более удобному виду
    :param paresedXml: XML преобразованный в JSON
    :return: справочник с настройками
    """
    if not paresedXml:
        return {}
    result = {}
    for prop in paresedXml["objects"]["object"]["property"]:
        if prop["name"] == "address":
            result["host"] = prop["map"]["entry"]["key"]
            result["port"] = prop["map"]["entry"]["value"]
        else:
            result[prop["name"]] = (prop["value"] if prop["value"] else "")
    return result


def testArtixBonus(pluginName):
    """
    Проверка соединения с бонусными сервисами для старых плагинов Artix
    :param pluginName: имя плагина (artixbonus или artixcertificate)
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    settings = parseArtixBonusSettings(requestSettings(pluginName, True))
    if not settings:
        return 2
    address = getValue(settings, "host") + ":" + getValue(settings, "port")
    if address[:4] != "http":
        address = "http://" + address
    timeout = getValue(settings, "connectTimeout")
    timeout = 10 if not timeout else int(timeout) / 1000
    try:
        resp = requests.get(address + "/ACC/rest/v1/version", timeout=timeout)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


def testArtixCoupon():
    """
    Проверка соединения с сервисом купонов Artix
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    if not pluginEnabled("OnlineCoupon"):
        return 2
    settings = requestSettings("OnlineCoupon")
    address = getValue(settings, "host") + ":" + getValue(settings, "port")
    if address[:4] != "http":
        address = "http://" + address
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    try:
        resp = requests.get(address + "/ACC/rest/v1/version", timeout=timeout)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


def testOnlineCards():
    """
    Проверка соединения с сервисом онлайн карт
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    settings = requestSettings("ArtixBonus.OnlineCards")
    if not checkLogical(getValue(settings, "enable")):
        return 2
    url = urlparse(getValue(settings, "url"))
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    try:
        resp = requests.get(url._replace(path="/version").geturl(), timeout=timeout)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


def testDiscountCounters():
    """
    Проверка соединения с сервисом счетчиков
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    settings = requestSettings("DiscountCounters")
    if not checkLogical(getValue(settings, "enable")):
        return 2
    address = getValue(settings, "host") + ":" + getValue(settings, "port")
    if address[:4] != "http":
        address = "http://" + address
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    try:
        resp = requests.get(address + "/version", timeout=timeout)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


def testCardIssuance():
    """
    Проверка соединения с сервисом выдачи карт
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    if not pluginEnabled("CardIssuance"):
        return 2
    settings = requestSettings("CardIssuance")
    url = urlparse(getValue(settings, "url"))
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    try:
        resp = requests.get(url._replace(path="/version").geturl(), timeout=timeout)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


"""
Скипт возвращает одно из 4 значений:
0 - связть с сервисом есть
1 - связи с сервисом нет
2 - плагин на кассе выключен
3 - неизвестное название плагина
"""
pluginName = sys.argv[1].lower()
if pluginName == "artixbonus" or pluginName == "artixcertificate":
    print(testArtixBonus(pluginName))
elif pluginName == "onlinecoupon":
    print(testArtixCoupon())
elif pluginName == "onlinecards":
    print(testOnlineCards())
elif pluginName == "discountcounters":
    print(testDiscountCounters())
elif pluginName == "cardissuance":
    print(testCardIssuance())
else:
    print(3)
